@@ -1,3 +1,10 @@
+import torch
+from packaging import version
+
+if version.parse(torch.__version__) >= version.parse('2.0.0'):
+    from einops._torch_specific import allow_ops_in_compiled_graph
+    allow_ops_in_compiled_graph()
+
 from audiolm_pytorch.audiolm_pytorch import AudioLM
 from audiolm_pytorch.soundstream import SoundStream, AudioLMSoundStream, MusicLMSoundStream
 from audiolm_pytorch.encodec import EncodecWrapper
