@@ -1293,7 +1293,7 @@ class CoarseTransformerWrapper(nn.Module):
 
         self.semantic_cross_entropy_loss_weight = semantic_cross_entropy_loss_weight
 
-        self.num_coarse_quantizers = transformer.num_coarse_quantizers
+        self.num_coarse_quantizers = transformer.num_coarse_quantizers * codec.rq_groups
         self.semantic_eos_id = transformer.semantic_eos_id
         self.coarse_eos_id = transformer.coarse_eos_id
 
@@ -1411,8 +1411,8 @@ class CoarseTransformerWrapper(nn.Module):
             with torch.no_grad():
                 self.codec.eval()
                 _, indices, _ = self.codec(raw_wave_for_codec, return_encoded = True)
-                batch = raw_wave.shape[0]
-                num_timesteps = raw_wave.shape[1]
+                batch = raw_wave_for_codec.shape[0]
+                num_timesteps = raw_wave_for_codec.shape[1]
                 num_frames = int(num_timesteps / self.codec.seq_len_multiple_of)
                 assert indices.shape[0] == batch and indices.shape[1] == num_frames, \
                     f'Expected indices to have shape (batch, num_frames, num_coarse_quantizers + num_fine_quantizers), but got {indices.shape}'
@@ -1463,12 +1463,16 @@ class CoarseTransformerWrapper(nn.Module):
         coarse_logits, semantic_logits = map(lambda t: maybe(rearrange)(t, 'b n c -> b c n'), (coarse_logits, semantic_logits))
 
         if self.unique_consecutive:
-            num_coarse_logits, num_semantic_logits = coarse_labels.numel(), (semantic_labels != self.pad_id).sum()
+            num_coarse_logits, _num_semantic_logits = coarse_labels.numel(), (semantic_labels != self.pad_id).sum()
         else:
-            num_coarse_logits, num_semantic_logits = coarse_logits.shape[-1], semantic_logits.shape[-1]
+            num_coarse_logits, _num_semantic_logits = coarse_logits.shape[-1], semantic_logits.shape[-1]
 
         semantic_loss = 0.
+        num_semantic_logits = 0
+
         if self.semantic_cross_entropy_loss_weight > 0 and exists(semantic_logits):
+            num_semantic_logits = _num_semantic_logits
+
             semantic_loss = F.cross_entropy(
                 semantic_logits,
                 semantic_labels,
@@ -1506,11 +1510,11 @@ class FineTransformerWrapper(nn.Module):
 
         assert not (exists(audio_conditioner) and not transformer.has_condition), 'if conditioning on audio embeddings from mulan, transformer has_condition must be set to True'
 
-        self.num_fine_quantizers = transformer.num_fine_quantizers
-        self.num_coarse_quantizers = transformer.num_coarse_quantizers
+        self.num_fine_quantizers = transformer.num_fine_quantizers * codec.rq_groups
+        self.num_coarse_quantizers = transformer.num_coarse_quantizers * codec.rq_groups
 
         if exists(codec):
-            assert (self.num_fine_quantizers + self.num_coarse_quantizers) == codec.num_quantizers, 'number of fine and coarse quantizers on fine transformer must add up to total number of quantizers on codec'
+            assert (self.num_fine_quantizers + self.num_coarse_quantizers) == (codec.num_quantizers * codec.rq_groups), 'number of fine and coarse quantizers on fine transformer must add up to total number of quantizers on codec'
 
         self.eos_id = transformer.eos_id
 
