@@ -4,6 +4,7 @@ from pathlib import Path
 
 from functools import partial, wraps
 from itertools import zip_longest
+from typing import Optional
 
 import torch
 from torch import nn, einsum
@@ -16,7 +17,7 @@ from torchaudio.functional import resample
 
 from einops import rearrange, reduce, pack, unpack
 
-from vector_quantize_pytorch import ResidualVQ
+from vector_quantize_pytorch import GroupedResidualVQ
 
 from local_attention import LocalMHA
 from local_attention.transformer import FeedForward, DynamicPositionBias
@@ -200,8 +201,11 @@ def ComplexSTFTResidualUnit(chan_in, chan_out, strides):
     paddings = tuple(map(lambda t: t // 2, kernel_sizes))
 
     return nn.Sequential(
-        ComplexConv2d(chan_in, chan_in, 3, padding = 1),
-        ModReLU(),
+        Residual(Sequential(
+            ComplexConv2d(chan_in, chan_in, 3, padding = 1),
+            ModReLU(),
+            ComplexConv2d(chan_in, chan_in, 3, padding = 1)
+        )),
         ComplexConv2d(chan_in, chan_out, kernel_sizes, stride = strides, padding = paddings)
     )
 
@@ -301,16 +305,18 @@ class Residual(nn.Module):
         return self.fn(x, **kwargs) + x
 
 class CausalConv1d(nn.Module):
-    def __init__(self, chan_in, chan_out, kernel_size, **kwargs):
+    def __init__(self, chan_in, chan_out, kernel_size, pad_mode = 'reflect', **kwargs):
         super().__init__()
         kernel_size = kernel_size
         dilation = kwargs.get('dilation', 1)
-        self.causal_padding = dilation * (kernel_size - 1)
+        stride = kwargs.get('stride', 1)
+        self.pad_mode = pad_mode
+        self.causal_padding = dilation * (kernel_size - 1) + (1 - stride)
 
         self.conv = nn.Conv1d(chan_in, chan_out, kernel_size, **kwargs)
 
     def forward(self, x):
-        x = F.pad(x, (self.causal_padding, 0))
+        x = F.pad(x, (self.causal_padding, 0), mode = self.pad_mode)
         return self.conv(x)
 
 class CausalConvTranspose1d(nn.Module):
@@ -328,18 +334,18 @@ class CausalConvTranspose1d(nn.Module):
 
         return out
 
-def ResidualUnit(chan_in, chan_out, dilation, kernel_size = 7, squeeze_excite = False):
+def ResidualUnit(chan_in, chan_out, dilation, kernel_size = 7, squeeze_excite = False, pad_mode = 'reflect'):
     return Residual(Sequential(
-        CausalConv1d(chan_in, chan_out, kernel_size, dilation = dilation),
+        CausalConv1d(chan_in, chan_out, kernel_size, dilation = dilation, pad_mode = pad_mode),
         nn.ELU(),
-        CausalConv1d(chan_out, chan_out, 1),
+        CausalConv1d(chan_out, chan_out, 1, pad_mode = pad_mode),
         nn.ELU(),
         SqueezeExcite(chan_out) if squeeze_excite else None
     ))
 
-def EncoderBlock(chan_in, chan_out, stride, cycle_dilations = (1, 3, 9), squeeze_excite = False):
+def EncoderBlock(chan_in, chan_out, stride, cycle_dilations = (1, 3, 9), squeeze_excite = False, pad_mode = 'reflect'):
     it = cycle(cycle_dilations)
-    residual_unit = partial(ResidualUnit, squeeze_excite = squeeze_excite)
+    residual_unit = partial(ResidualUnit, squeeze_excite = squeeze_excite, pad_mode = pad_mode)
 
     return nn.Sequential(
         residual_unit(chan_in, chan_in, next(it)),
@@ -348,12 +354,12 @@ def EncoderBlock(chan_in, chan_out, stride, cycle_dilations = (1, 3, 9), squeeze
         CausalConv1d(chan_in, chan_out, 2 * stride, stride = stride)
     )
 
-def DecoderBlock(chan_in, chan_out, stride, cycle_dilations = (1, 3, 9), squeeze_excite = False):
+def DecoderBlock(chan_in, chan_out, stride, cycle_dilations = (1, 3, 9), squeeze_excite = False, pad_mode = 'reflect'):
     even_stride = (stride % 2 == 0)
     padding = (stride + (0 if even_stride else 1)) // 2
     output_padding = 0 if even_stride else 1
 
-    residual_unit = partial(ResidualUnit, squeeze_excite = squeeze_excite)
+    residual_unit = partial(ResidualUnit, squeeze_excite = squeeze_excite, pad_mode = pad_mode)
 
     it = cycle(cycle_dilations)
     return nn.Sequential(
@@ -399,6 +405,15 @@ class LocalTransformer(nn.Module):
 
         return x
 
+class FiLM(nn.Module):
+    def __init__(self, dim, dim_cond):
+        super().__init__()
+        self.to_cond = nn.Linear(dim_cond, dim * 2)
+
+    def forward(self, x, cond):
+        gamma, beta = self.to_cond(cond).chunk(2, dim = -1)
+        return x * gamma + beta
+
 class SoundStream(nn.Module):
     def __init__(
         self,
@@ -412,6 +427,7 @@ class SoundStream(nn.Module):
         rq_commitment_weight = 1.,
         rq_ema_decay = 0.95,
         rq_quantize_dropout_multiple_of = 1,
+        rq_groups = 1,
         input_channels = 1,
         discr_multi_scales = (1, 0.5, 0.25),
         stft_normalized = False,
@@ -421,7 +437,7 @@ class SoundStream(nn.Module):
         multi_spectral_n_ffts = 512,
         multi_spectral_n_mels = 64,
         recon_loss_weight = 1.,
-        multi_spectral_recon_loss_weight = 1.,
+        multi_spectral_recon_loss_weight = 1e-5,
         adversarial_loss_weight = 1.,
         feature_loss_weight = 100,
         quantize_dropout_cutoff_index = 1,
@@ -434,7 +450,9 @@ class SoundStream(nn.Module):
         attn_xpos_scale_base = None,
         attn_dynamic_pos_bias = False,
         squeeze_excite = False,
-        complex_stft_discr_logits_abs = True
+        complex_stft_discr_logits_abs = True,
+        pad_mode = 'reflect',
+        stft_discriminator: Optional[nn.Module] = None  # can pass in own stft discriminator
     ):
         super().__init__()
 
@@ -459,12 +477,12 @@ class SoundStream(nn.Module):
         encoder_blocks = []
 
         for ((chan_in, chan_out), layer_stride) in zip(chan_in_out_pairs, strides):
-            encoder_blocks.append(EncoderBlock(chan_in, chan_out, layer_stride, enc_cycle_dilations, squeeze_excite))
+            encoder_blocks.append(EncoderBlock(chan_in, chan_out, layer_stride, enc_cycle_dilations, squeeze_excite, pad_mode))
 
         self.encoder = nn.Sequential(
-            CausalConv1d(input_channels, channels, 7),
+            CausalConv1d(input_channels, channels, 7, pad_mode = pad_mode),
             *encoder_blocks,
-            CausalConv1d(layer_channels[-1], codebook_dim, 3)
+            CausalConv1d(layer_channels[-1], codebook_dim, 3, pad_mode = pad_mode)
         )
 
         attn_kwargs = dict(
@@ -481,12 +499,20 @@ class SoundStream(nn.Module):
 
         self.encoder_attn = LocalTransformer(**attn_kwargs) if use_local_attn else None
 
+        self.encoder_film = FiLM(codebook_dim, dim_cond = 2)
+
         self.num_quantizers = rq_num_quantizers
 
-        self.rq = ResidualVQ(
+        self.codebook_dim = codebook_dim
+        self.codebook_size = codebook_size
+
+        self.rq_groups = rq_groups
+
+        self.rq = GroupedResidualVQ(
             dim = codebook_dim,
             num_quantizers = rq_num_quantizers,
             codebook_size = codebook_size,
+            groups = rq_groups,
             decay = rq_ema_decay,
             commitment_weight = rq_commitment_weight,
             quantize_dropout_multiple_of = rq_quantize_dropout_multiple_of,
@@ -496,17 +522,19 @@ class SoundStream(nn.Module):
             quantize_dropout_cutoff_index = quantize_dropout_cutoff_index
         )
 
+        self.decoder_film = FiLM(codebook_dim, dim_cond = 2)
+
         self.decoder_attn = LocalTransformer(**attn_kwargs) if use_local_attn else None
 
         decoder_blocks = []
 
         for ((chan_in, chan_out), layer_stride) in zip(reversed(chan_in_out_pairs), reversed(strides)):
-            decoder_blocks.append(DecoderBlock(chan_out, chan_in, layer_stride, dec_cycle_dilations, squeeze_excite))
+            decoder_blocks.append(DecoderBlock(chan_out, chan_in, layer_stride, dec_cycle_dilations, squeeze_excite, pad_mode))
 
         self.decoder = nn.Sequential(
-            CausalConv1d(codebook_dim, layer_channels[-1], 7),
+            CausalConv1d(codebook_dim, layer_channels[-1], 7, pad_mode = pad_mode),
             *decoder_blocks,
-            CausalConv1d(channels, input_channels, 7)
+            CausalConv1d(channels, input_channels, 7, pad_mode = pad_mode)
         )
 
         # discriminators
@@ -516,10 +544,13 @@ class SoundStream(nn.Module):
         discr_rel_factors = [int(s1 / s2) for s1, s2 in zip(discr_multi_scales[:-1], discr_multi_scales[1:])]
         self.downsamples = nn.ModuleList([nn.Identity()] + [nn.AvgPool1d(2 * factor, stride = factor, padding = factor) for factor in discr_rel_factors])
 
-        self.stft_discriminator = ComplexSTFTDiscriminator(
-            stft_normalized = stft_normalized,
-            logits_abs = complex_stft_discr_logits_abs  # whether to output as abs() or use view_as_real
-        )
+        self.stft_discriminator = stft_discriminator
+
+        if not exists(self.stft_discriminator):
+            self.stft_discriminator = ComplexSTFTDiscriminator(
+                stft_normalized = stft_normalized,
+                logits_abs = complex_stft_discr_logits_abs  # whether to output as abs() or use view_as_real
+            )
 
         # multi spectral reconstruction
 
@@ -559,13 +590,25 @@ class SoundStream(nn.Module):
 
         self.register_buffer('zero', torch.tensor([0.]), persistent = False)
 
+    @property
+    def device(self):
+        return next(self.parameters()).device
+
     @property
     def configs(self):
         return pickle.loads(self._configs)
 
     def decode_from_codebook_indices(self, quantized_indices):
+        quantized_indices = rearrange(quantized_indices, 'b n (g q) -> g b n q', g = self.rq_groups)
+
         codes = self.rq.get_codes_from_indices(quantized_indices)
-        x = reduce(codes, 'q ... -> ...', 'sum')
+        x = reduce(codes, 'g q b n d -> b n (g d)', 'sum')
+
+        return self.decode(x)
+
+    def decode(self, x, quantize = False):
+        if quantize:
+            x, *_ = self.rq(x)
 
         x = self.decoder_attn(x)
         x = rearrange(x, 'b n c -> b c n')
@@ -624,34 +667,56 @@ class SoundStream(nn.Module):
             *self.encoder.parameters(),
             *self.decoder.parameters(),
             *(self.encoder_attn.parameters() if exists(self.encoder_attn) else []),
-            *(self.decoder_attn.parameters() if exists(self.decoder_attn) else [])
+            *(self.decoder_attn.parameters() if exists(self.decoder_attn) else []),
+            *self.encoder_film.parameters(),
+            *self.decoder_film.parameters()
         ]
 
     @property
     def seq_len_multiple_of(self):
         return functools.reduce(lambda x, y: x * y, self.strides)
 
-    def forward(
+    def process_input(
         self,
         x,
-        return_encoded = False,
-        return_discr_loss = False,
-        return_discr_losses_separately = False,
-        return_loss_breakdown = False,
-        return_recons_only = False,
         input_sample_hz = None,
-        apply_grad_penalty = False
+        curtail_from_left = False
     ):
         x, ps = pack([x], '* n')
 
         if exists(input_sample_hz):
             x = resample(x, input_sample_hz, self.target_sample_hz)
 
-        x = curtail_to_multiple(x, self.seq_len_multiple_of)
+        x = curtail_to_multiple(x, self.seq_len_multiple_of, from_left = curtail_from_left)
 
         if x.ndim == 2:
             x = rearrange(x, 'b n -> b 1 n')
 
+        return x, ps
+
+    def forward(
+        self,
+        x,
+        target = None,
+        is_denoising = None, # if you want to learn film conditioners that teach the soundstream to denoise - target would need to be passed in above
+        return_encoded = False,
+        return_discr_loss = False,
+        return_discr_losses_separately = False,
+        return_loss_breakdown = False,
+        return_recons_only = False,
+        input_sample_hz = None,
+        apply_grad_penalty = False,
+        curtail_from_left = False
+    ):
+        assert not (exists(is_denoising) and not exists(target))
+
+        process_input = partial(self.process_input, input_sample_hz = input_sample_hz, curtail_from_left = curtail_from_left)
+
+        x, ps = process_input(x)
+
+        if exists(target):
+            target, _ = process_input(target)
+
         orig_x = x.clone()
 
         x = self.encoder(x)
@@ -661,16 +726,24 @@ class SoundStream(nn.Module):
         if exists(self.encoder_attn):
             x = self.encoder_attn(x)
 
+        if exists(is_denoising):
+            denoise_input = torch.tensor([is_denoising, not is_denoising], dtype = x.dtype, device = self.device) # [1, 0] for denoise, [0, 1] for not denoising
+            x = self.encoder_film(x, denoise_input)
+
         x, indices, commit_loss = self.rq(x)
 
+        if return_encoded:
+            indices = rearrange(indices, 'g b n q -> b n (g q)')
+            return x, indices, commit_loss
+
+        if exists(is_denoising):
+            x = self.decoder_film(x, denoise_input)
+
         if exists(self.decoder_attn):
             x = self.decoder_attn(x)
 
         x = rearrange(x, 'b n c -> b c n')
 
-        if return_encoded:
-            return x, indices, commit_loss
-
         recon_x = self.decoder(x)
 
         if return_recons_only:
@@ -735,7 +808,9 @@ class SoundStream(nn.Module):
 
         # recon loss
 
-        recon_loss = F.mse_loss(orig_x, recon_x)
+        target = default(target, orig_x)  # target can also be passed in, in the case of denoising
+
+        recon_loss = F.mse_loss(target, recon_x)
 
         # multispectral recon loss - eq (4) and (5) in https://arxiv.org/abs/2107.03312
 
