@@ -1,3 +1,4 @@
+import re
 from math import sqrt
 import copy
 from random import choice
@@ -111,6 +112,14 @@ def determine_types(data, config):
 
     return tuple(output)
 
+def checkpoint_num_steps(checkpoint_path):
+    """Returns the number of steps trained from a checkpoint based on the filename.
+
+    Filename format assumed to be something like "/path/to/semantic.transformer.20000.pt" which is
+    for 20k train steps. Returns 20000 in that case.
+    """
+    return int(re.findall(r'\d+', str(checkpoint_path))[-1])
+
 # main trainer class
 
 class SoundStreamTrainer(nn.Module):
@@ -122,7 +131,7 @@ class SoundStreamTrainer(nn.Module):
         num_train_steps: int,
         batch_size: int,
         data_max_length: int = None,
-        data_max_length_seconds: float = None,
+        data_max_length_seconds: Union[int, float] = None,
         folder: str = None,
         train_dataloader: DataLoader = None,
         val_dataloader: DataLoader = None,
@@ -342,6 +351,8 @@ class SoundStreamTrainer(nn.Module):
         for key, _ in self.multiscale_discriminator_iter():
             discr_optim = getattr(self, key)
             discr_optim.load_state_dict(pkg[key])
+        # + 1 to start from the next step and avoid overwriting the last checkpoint
+        self.steps = torch.tensor([checkpoint_num_steps(path) + 1], device=self.device)
 
     def multiscale_discriminator_iter(self):
         for ind, discr in enumerate(self.unwrapped_soundstream.discriminators):
@@ -668,6 +679,9 @@ class SemanticTransformerTrainer(nn.Module):
         transformer = self.accelerator.unwrap_model(self.transformer)
         transformer.load_state_dict(pkg['model'])
         self.optim.load_state_dict(pkg['optim'])
+        # + 1 to start from the next step and avoid overwriting the last checkpoint
+        self.steps = torch.tensor([checkpoint_num_steps(path) + 1], device=self.device)
+
 
     def print(self, msg):
         self.accelerator.print(msg)
@@ -918,6 +932,9 @@ class CoarseTransformerTrainer(nn.Module):
         transformer.load_state_dict(pkg['model'])
 
         self.optim.load_state_dict(pkg['optim'])
+        # + 1 to start from the next step and avoid overwriting the last checkpoint
+        self.steps = torch.tensor([checkpoint_num_steps(path) + 1], device=self.device)
+
 
     def print(self, msg):
         self.accelerator.print(msg)
@@ -1162,6 +1179,9 @@ class FineTransformerTrainer(nn.Module):
         transformer.load_state_dict(pkg['model'])
 
         self.optim.load_state_dict(pkg['optim'])
+        # + 1 to start from the next step and avoid overwriting the last checkpoint
+        self.steps = torch.tensor([checkpoint_num_steps(path) + 1], device=self.device)
+
 
     def print(self, msg):
         self.accelerator.print(msg)
